/* Copyright 1993 Laboratoire de Teleinformatique, Ecole Polytechnique    */
/* Federale de Lausanne.  This program may be used, copied, modified, and */
/* redistributed freely for noncommercial purposes so long as this notice */
/* remains intact.                                                        */

#include <memory.h>

#define WindowRange  8
#define WindowSize   3
#define T1           500
#define TextLen      81

typedef enum { EXTRACT, NOEXTRACT } extract_flags;

typedef struct MessageType *TMessagePtr;

struct MessageType {
  TMessagePtr     next;
  long unsigned   date;
  int             NR,NS;
  char            text[TextLen];
};

inline void *MakeACopy(void *a,unsigned sz) {return memcpy(malloc(sz),a,(int)sz);}

/*---------------     Queueing  Management     ------------------*/

class Queue  {

private:
  TMessagePtr first,middle,last;
  
public:

  /*
   * Put message after the others into the queue in waiing for
   * sending to LLP mode
   */
  void PutMessage( TMessagePtr message );
  
  /*
   * return the reference to the oldest message in the queue waiting
   * for sending to LLP. After this call the message is still in the queue.
   */
  TMessagePtr NextToSend();
  
  /*
   * register departure time of the oldest message waiting for sending to
   * to network
   */
  void RecordDepartureTimeOfNextToSend( unsigned long DepartTime );
  
  /*
   * change mode of the oldest message waiting for sending to network,
   * to the state waiting for an acknowledgment
   */
  void UpdateNextToSend();
  
  /*
   * all messages already sent to network waiting for acknowledgments
   * must be marked: unsent to network waiting for sending to LLP
   */
  void ResetNextToSend();

  /*
   * return the timeout value of the oldest message sent to network and waiting
   * for acknowledgment if existing one else return 0.
   */
  unsigned long OldestWaitingAckMessageTime();
  
  /*
   * return the reference to oldest message sent to network and waiting
   * for acknowledgment if existing one else return NULL.
   * if flag is set to EXTRACT, the message will be extracted from the queue.
   */
  TMessagePtr OldestWaitingAckMessage( extract_flags flag );

  /*
   * Destroy the queue
   */
  ~Queue();
  
  /*
   * Create an empty queue
   */
  Queue ();
};

/* -------------------------- FIFO Management -------------------------- */

class FiFo {
  TMessagePtr first,last;
 
public:

  /*
   * Create an empty FiFo
   */
  FiFo ();

  /*
   * Destroy the FiFo
   */
  ~FiFo();
  
  /*
   * put message into the fifo
   */
  void put( TMessagePtr message );

  /*
   * return the reference of the first message from the fifo
   * NULL if empty
   * if flag is set to EXTRACT, the message will be extracted from the fifo
   */
  TMessagePtr get( extract_flags flag );

  /*
   * return 1 if fifo is empty, 0 otherwise
   */
  int empty();

};
  
